@@ -1,5 +1,5 @@
 #!/usr/bin/env perl
-# $Id: install-menu-text.pl 43566 2017-03-21 17:08:03Z karl $
+# $Id: install-menu-text.pl 44571 2017-06-12 13:23:16Z siepo $
 # install-menu-txt.pl
 #
 # Copyright 2007-2017 Norbert Preining, Karl Berry
@@ -12,6 +12,7 @@
 use vars qw(@::end_install_hook $::opt_no_cls);
 
 our %vars;
+our $opt_in_place;
 our $tlpdb;
 our @media_available;
 our $media;
@@ -32,10 +33,14 @@ my $ptoggle_alert = "\n".
     "   !! Portable option changed;\n" .
     "   !! Directories have been reinitialized!\n";
 
-# issue welcome message on end of installation
-push @::end_install_hook,
-    sub { if (win32()) { print TeXLive::TLUtils::welcome(); }
-          else { print TeXLive::TLUtils::welcome_paths(); } };
+# @fileassocdesc also defined in install-tl
+$::fileassocdesc[0] = "None";
+$::fileassocdesc[1] = "Only new";
+$::fileassocdesc[2] = "All";
+
+$::deskintdesc[0] = "None";
+$::deskintdesc[1] = "Menu shortcuts";
+if (win32() && is_seven()) { $::deskintdesc[2] = "Launcher"; }
 
 sub clear_screen {
   return 0 if ($::opt_no_cls);
@@ -47,9 +52,24 @@ sub string_to_list {
   return split(//, $string);
 }
 
-sub button {
+# ___, defined in install-tl, replaces the GUI translating function
+*::__ = \&::___;
+
+sub button { # for main menu: 1 char
   my $val=shift;
-  return ($val)? '[X]':'[ ]';
+  my $vals=shift; # array ref to descriptions of possible values
+  if (defined $vals) {
+    return "[$val]";
+  } else { # just yes/no
+    return ($val)? '[X]':'[ ]';
+  }
+}
+
+sub obutton { # for options menu: more chars allowed
+  my $val=shift;
+  my $vals=shift; # array ref to descriptions of possible values
+  $vals = [' ', 'X'] unless defined $vals;
+  return "[$$vals[$val]]";
 }
 
 sub hbar {
@@ -58,7 +78,11 @@ sub hbar {
 
 sub toggle {
   my $var=shift;
-  $vars{$var} = ($vars{$var} ? 0 : 1);
+  my $vals=shift; # array ref to descriptions of possible values
+  $vals = [' ', 'X'] unless defined $vals;
+  my $n_vals = @$vals;
+  $vars{$var} += 1;
+  $vars{$var} = 0 unless $vars{$var} < $n_vals;
 }
 
 sub menu_head {
@@ -74,10 +98,11 @@ sub other_options {
     '+' => 'select all',
     'H' => 'help',
     'R' => 'return to main menu',
+    'P' => 'save installation profile to \'texlive.profile\' and exit',
     'Q' => 'quit'
       );
 
-  $opts{'I'}=$vars{'portable'} ? 'start portable installation' :
+  $opts{'I'}=$vars{'instopt_portable'} ? 'start portable installation' :
     'start installation to hard disk';
 
   print "\nActions:";
@@ -487,7 +512,7 @@ sub directories_menu
   }
   my $texmfdir = $vars{'TEXDIR'} .
     ($vars{'TEXDIR'} =~ /\/$/ ? 'texmf-dist' : '/texmf-dist');
-  if (!$vars{'in_place'}) {
+  if (!$opt_in_place) {
     print <<"EOF";
  <1> TEXDIR:       $vars{'TEXDIR'}
      support tree: $texmfdir
@@ -498,7 +523,7 @@ EOF
      support tree: $texmfdir
 EOF
   }
-  if (!$vars{'portable'}) {
+  if (!$vars{'instopt_portable'}) {
     print <<"EOF";
 
  <2> TEXMFLOCAL:     $vars{'TEXMFLOCAL'}
@@ -521,7 +546,7 @@ EOF
   other_options qw(R Q);
   my $answer = prompt 'Enter command';
 
-  if ("\u$answer" eq '1' and !$vars{'in_place'}) {
+  if ("\u$answer" eq '1' and !$opt_in_place) {
     print "New value for TEXDIR [$vars{'TEXDIR'}]: ";
     $answer = &input_dirname ();
     $vars{'TEXDIR'} = $answer if $answer ne "";
@@ -540,37 +565,37 @@ EOF
     }
     return $command{'self'};
 
-  } elsif ("\u$answer" eq '2' and !$vars{'portable'}) {
+  } elsif ("\u$answer" eq '2' and !$vars{'instopt_portable'}) {
     print "New value for TEXMFLOCAL [$vars{'TEXMFLOCAL'}]: ";
     $answer = &input_dirname ();
     $vars{'TEXMFLOCAL'} = $answer if $answer ne "";
     return $command{'self'};
 
-  } elsif ("\u$answer" eq '3' and !$vars{'portable'}) {
+  } elsif ("\u$answer" eq '3' and !$vars{'instopt_portable'}) {
     print "New value for TEXMFSYSVAR [$vars{'TEXMFSYSVAR'}]: ";
     $answer = &input_dirname ();
     $vars{'TEXMFSYSVAR'} = $answer if $answer ne "";
     return $command{'self'};
 
-  } elsif ("\u$answer" eq '4' and !$vars{'portable'}) {
+  } elsif ("\u$answer" eq '4' and !$vars{'instopt_portable'}) {
     print "New value for TEXMFSYSCONFIG [$vars{'TEXMFSYSCONFIG'}]: ";
     $answer = &input_dirname ();
     $vars{'TEXMFSYSCONFIG'} = $answer if $answer ne "";
     return $command{'self'};
 
-  } elsif ("\u$answer" eq '5' and !$vars{'portable'}) {
+  } elsif ("\u$answer" eq '5' and !$vars{'instopt_portable'}) {
     print "New value for TEXMFVAR [$vars{'TEXMFVAR'}]: ";
     $answer = &input_dirname ("noexpansion");
     $vars{'TEXMFVAR'} = $answer if $answer ne "";
     return $command{'self'};
 
-  } elsif ("\u$answer" eq '6' and !$vars{'portable'}) {
+  } elsif ("\u$answer" eq '6' and !$vars{'instopt_portable'}) {
     print "New value for TEXMFCONFIG [$vars{'TEXMFCONFIG'}]: ";
     $answer = &input_dirname ("noexpansion");
     $vars{'TEXMFCONFIG'} = $answer if $answer ne "";
     return $command{'self'};
 
-  } elsif ("\u$answer" eq '7' and !$vars{'portable'}) {
+  } elsif ("\u$answer" eq '7' and !$vars{'instopt_portable'}) {
     print "New value for TEXMFHOME [$vars{'TEXMFHOME'}]: ";
     $answer = &input_dirname ("noexpansion");
     $vars{'TEXMFHOME'} = $answer if $answer ne "";
@@ -736,24 +761,25 @@ sub help_menu {
 
 
 sub options_menu {
-  my $b_path=button($vars{'option_path'});
-  my $b_doc=button($vars{'option_doc'});
-  my $b_src=button($vars{'option_src'});
-  my $b_fmt=button($vars{'option_fmt'});
-  my $b_letter=button($vars{'option_letter'});
-  my $b_adjustrepo=button($vars{'option_adjustrepo'});
-  my $b_deskint=button($vars{'option_desktop_integration'});
-  my $b_admin=button($vars{'option_w32_multi_user'});
-  my $b_addoneditor=button($vars{'collection-texworks'});
-  my $b_restricted=button($vars{'option_write18_restricted'});
+  my $b_path=obutton($vars{'instopt_adjustpath'});
+  my $b_doc=obutton($vars{'tlpdbopt_install_docfiles'});
+  my $b_src=obutton($vars{'tlpdbopt_install_srcfiles'});
+  my $b_fmt=obutton($vars{'tlpdbopt_create_formats'});
+  my $b_letter=obutton($vars{'instopt_letter'});
+  my $b_adjustrepo=obutton($vars{'instopt_adjustrepo'});
+  my $b_deskint=obutton(
+    $vars{'tlpdbopt_desktop_integration'}, \@::deskintdesc);
+  my $b_admin=obutton($vars{'tlpdbopt_w32_multi_user'});
+  my $b_addoneditor=obutton($vars{'collection-texworks'});
+  my $b_restricted=obutton($vars{'instopt_write18_restricted'});
 
-  my $sys_bin=$vars{'option_sys_bin'};
-  my $sys_man=$vars{'option_sys_man'};
-  my $sys_info=$vars{'option_sys_info'};
+  my $sys_bin=$vars{'tlpdbopt_sys_bin'};
+  my $sys_man=$vars{'tlpdbopt_sys_man'};
+  my $sys_info=$vars{'tlpdbopt_sys_info'};
 
-  my $t_sys_bin=($vars{'option_path'})? $vars{'option_sys_bin'}:'';
-  my $t_sys_man=($vars{'option_path'})? $vars{'option_sys_man'}:'';
-  my $t_sys_info=($vars{'option_path'})? $vars{'option_sys_info'}:'';
+  my $t_sys_bin=($vars{'instopt_adjustpath'})? $vars{'tlpdbopt_sys_bin'}:'';
+  my $t_sys_man=($vars{'instopt_adjustpath'})? $vars{'tlpdbopt_sys_man'}:'';
+  my $t_sys_info=($vars{'instopt_adjustpath'})? $vars{'tlpdbopt_sys_info'}:'';
 
   my %command=(
     'self' => \&options_menu,
@@ -770,13 +796,13 @@ sub options_menu {
  <F> create format files:                      $b_fmt
 EOF
 ;
-  if ($vars{'doc_splitting_supported'} and !$vars{'in_place'}) {
+  if ($vars{'doc_splitting_supported'} and !$opt_in_place) {
     print " <D> install font/macro doc tree:              $b_doc\n";
   }
-  if ($vars{'src_splitting_supported'} and !$vars{'in_place'}) {
+  if ($vars{'src_splitting_supported'} and !$opt_in_place) {
     print " <S> install font/macro source tree:           $b_src\n";
   }
-  if (!$vars{'portable'}) {
+  if (!$vars{'instopt_portable'}) {
     if (unix() || $::opt_all_options) {
       print <<"EOF";
  <L> create symlinks in standard directories:  $b_path
@@ -791,9 +817,11 @@ EOF
 EOF
 ;
     }
-    if (win32() || $::opt_all_options) {
-      print " <M> create Start menu shortcuts:              $b_deskint\n";
-      print " <N> update file associations:                 [$::fileassocdesc[$vars{'option_file_assocs'}]]\n";
+    if ((win32() && !$vars{'instopt_portable'}) || $::opt_all_options) {
+      print " <M> Start menu shortcuts / launcher:          ".obutton(
+          $vars{'tlpdbopt_desktop_integration'}, \@::deskintdesc)."\n";
+      print " <N> update file associations:                 ".obutton(
+        $vars{'tlpdbopt_file_assocs'}, \@::fileassocdesc)."\n";
       if ($::opt_all_options || TeXLive::TLWinGoo::admin()) {
         # if we are admin we allow normal user installation, too
         print " <U> make installation available to all users: $b_admin\n";
@@ -809,105 +837,113 @@ EOF
   other_options qw(R Q diskspace);
   my $answer = prompt 'Enter command';
 
-  # option_path
+  # instopt_adjustpath
 
   if (unix()) {
-    if (("\u$answer" eq 'L') and !$vars{'portable'}) {
+    if (("\u$answer" eq 'L') and !$vars{'instopt_portable'}) {
       my $home = getenv('HOME');
       $home = getenv('USERPROFILE') if (win32());
       $home ||= '~';
-      toggle 'option_path';
-      if ($vars{'option_path'}) {
+      toggle 'instopt_adjustpath';
+      if ($vars{'instopt_adjustpath'}) {
         print "New value for binary directory [$sys_bin]: ";
         chomp($answer=<STDIN>);
-        $vars{'option_sys_bin'} =  "$answer" if (length $answer);
-        $vars{'option_sys_bin'} =~ s@\\@/@g if (win32());
-        $vars{'option_sys_bin'} =~ s/^~/$home/;
-        if ($vars{'option_sys_bin'}=~/^(.*)\/bin$/) {
-          $vars{'option_sys_man'}="$1/man";
-          $vars{'option_sys_info'}="$1/info";
+        $vars{'tlpdbopt_sys_bin'} =  "$answer" if (length $answer);
+        $vars{'tlpdbopt_sys_bin'} =~ s@\\@/@g if (win32());
+        $vars{'tlpdbopt_sys_bin'} =~ s/^~/$home/;
+        if ($vars{'tlpdbopt_sys_bin'}=~/^(.*)\/bin$/) {
+          $vars{'tlpdbopt_sys_man'}="$1/man";
+          $vars{'tlpdbopt_sys_info'}="$1/info";
         }
-        print "New value for man directory    [$vars{'option_sys_man'}]: ";
+        print "New value for man directory    [$vars{'tlpdbopt_sys_man'}]: ";
         chomp($answer=<STDIN>);
-        $vars{'option_sys_man'}="$answer" if (length $answer);
-        $vars{'option_sys_man'} =~ s@\\@/@g if (win32());
-        $vars{'option_sys_man'} =~ s/^~/$home/;
+        $vars{'tlpdbopt_sys_man'}="$answer" if (length $answer);
+        $vars{'tlpdbopt_sys_man'} =~ s@\\@/@g if (win32());
+        $vars{'tlpdbopt_sys_man'} =~ s/^~/$home/;
 
-        print "New value for info directory   [$vars{'option_sys_info'}]: ";
+        print "New value for info directory   [$vars{'tlpdbopt_sys_info'}]: ";
         chomp($answer=<STDIN>);
-        $vars{'option_sys_info'}="$answer" if (length $answer);
-        $vars{'option_sys_info'} =~ s@\\@/@g if (win32());
-        $vars{'option_sys_info'} =~ s/^~/$home/;
+        $vars{'tlpdbopt_sys_info'}="$answer" if (length $answer);
+        $vars{'tlpdbopt_sys_info'} =~ s@\\@/@g if (win32());
+        $vars{'tlpdbopt_sys_info'} =~ s/^~/$home/;
       }
       return $command{'self'};
     }
   } else {
-    if (("\u$answer" eq 'L') and !$vars{'portable'}) {
+    if (("\u$answer" eq 'L') and !$vars{'instopt_portable'}) {
       my $home = getenv('HOME');
       $home = getenv('USERPROFILE') if (win32());
       $home ||= '~';
-      toggle 'option_path';
+      toggle 'instopt_adjustpath';
       return $command{'self'};
     }
   }
 
-  # option_desktop_integration, option_file_assocs
-
-  if (win32() || $::opt_all_options) {
-    if ("\u$answer" eq 'M' and !$vars{'portable'}) {
-      toggle 'option_desktop_integration';
-      return $command{'self'};
-
-    } elsif ("\u$answer" eq 'N' and !$vars{'portable'}) {
-      print "New value for file_assocs:\n";
-      print "  0 -- don't tweak the file associations\n";
-      print "  1 -- only add new file associations, don't overwrite old ones\n";
-      print "  2 -- always create file associations to TeX Live programs\n";
-      print "New value for file_assocs [$vars{'option_file_assocs'}]: ";
-      chomp (my $a = <STDIN>);
-      if ($a eq "0" || $a eq "1" || $a eq "2") {
-        $vars{'option_file_assocs'} = $a;
-      }
-      return $command{'self'};
-
-    } elsif ("\u$answer" eq 'U' and !$vars{'portable'}) {
-      toggle 'option_w32_multi_user';
-      return $command{'self'};
-    }
-  }
+#  # tlpdbopt_desktop_integration, tlpdbopt_file_assocs
+#
+#  if (win32() || $::opt_all_options) {
+#    if ("\u$answer" eq 'M' and !$vars{'instopt_portable'}) {
+#      toggle 'tlpdbopt_desktop_integration';
+#      return $command{'self'};
+#
+#    } elsif ("\u$answer" eq 'N' and !$vars{'instopt_portable'}) {
+#      print "New value for file_assocs:\n";
+#      print "  0 -- don't tweak the file associations\n";
+#      print "  1 -- only add new file associations, don't overwrite old ones\n";
+#      print "  2 -- always create file associations to TeX Live programs\n";
+#      print "New value for file_assocs [$vars{'tlpdbopt_file_assocs'}]: ";
+#      chomp (my $a = <STDIN>);
+#      if ($a eq "0" || $a eq "1" || $a eq "2") {
+#        $vars{'tlpdbopt_file_assocs'} = $a;
+#      }
+#      return $command{'self'};
+#
+#    } elsif ("\u$answer" eq 'U' and !$vars{'instopt_portable'}) {
+#      toggle 'tlpdbopt_w32_multi_user';
+#      return $command{'self'};
+#    }
+#  }
 
   # other options
 
-  if ("\u$answer" eq 'P') {
-    toggle 'option_letter';
+  if (("\u$answer" eq 'M') && !$vars{'instopt_portable'}) {
+    toggle ('tlpdbopt_desktop_integration', \@::deskintdesc);
+    return $command{'self'};
+
+  } elsif (("\u$answer" eq 'N') && !$vars{'instopt_portable'}) {
+    toggle ('tlpdbopt_file_assocs', \@::fileassocdesc);
+    return $command{'self'};
+
+  } elsif ("\u$answer" eq 'P') {
+    toggle 'instopt_letter';
     return $command{'self'};
 
   } elsif ("\u$answer" eq 'F') {
-    toggle 'option_fmt';
+    toggle 'tlpdbopt_create_formats';
     return $command{'self'};
 
   } elsif ("\u$answer" eq 'E') {
-    toggle 'option_write18_restricted';
+    toggle 'instopt_write18_restricted';
     return $command{'self'};
 
-  } elsif ("\u$answer" eq 'S' and !$vars{'in_place'}) {
-    toggle 'option_src';
+  } elsif ("\u$answer" eq 'S' and !$opt_in_place) {
+    toggle 'tlpdbopt_install_srcfiles';
     return $command{'self'};
 
-  } elsif ("\u$answer" eq 'D' and !$vars{'in_place'}) {
-    toggle 'option_doc';
+  } elsif ("\u$answer" eq 'D' and !$opt_in_place) {
+    toggle 'tlpdbopt_install_docfiles';
     return $command{'self'};
 
   } elsif (defined $command{"\u$answer"}) {
     return $command{"\u$answer"};
 
   } elsif (("\u$answer" eq 'W') && ($::opt_all_options || win32()) &&
-       !$vars{'in_place'} ) {
+       !$opt_in_place ) {
     toggle 'collection-texworks';
     return $command{'self'};
 
   } elsif ("\u$answer" eq 'Y' and $media ne "NET") {
-    toggle 'option_adjustrepo';
+    toggle 'instopt_adjustrepo';
     return $command{'self'};
 
   } else {
@@ -927,37 +963,46 @@ sub do_install {
 }
 
 sub toggle_portable {
-  if ($vars{'portable'}) {
-    $vars{'portable'} = 0;
-    $vars{'option_desktop_integration'} = 0;
-    $vars{'option_path'} = 0;
-    $vars{'option_file_assocs'} = 0;
-    $vars{'option_w32_multi_user'} = 0;
+  if ($vars{'instopt_portable'}) {
+    $vars{'instopt_portable'} = 0;
+    $vars{'tlpdbopt_desktop_integration'} = 0;
+    $vars{'instopt_adjustpath'} = 0;
+    $vars{'tlpdbopt_file_assocs'} = 0;
+    $vars{'tlpdbopt_w32_multi_user'} = 0;
   } else {
-    $vars{'portable'} = 1;
-    $vars{'option_desktop_integration'} = 1;
-    $vars{'option_path'} = 1;
-    $vars{'option_file_assocs'} = 1;
-    $vars{'option_w32_multi_user'} = 1;
+    $vars{'instopt_portable'} = 1;
+    $vars{'tlpdbopt_desktop_integration'} = 1;
+    $vars{'instopt_adjustpath'} = 1;
+    $vars{'tlpdbopt_file_assocs'} = 1;
+    $vars{'tlpdbopt_w32_multi_user'} = 1;
   }
   $portable_toggled = 1;
   set_texlive_default_dirs(); # this sub tests for portable and in_place
   main_menu;
 }
 
+sub callback_save_profile {
+  create_profile("texlive.profile");
+  print "Installation profile saved to 'texlive.profile', exiting.\n";
+  exit(0);
+}
+
 sub main_menu {
   my $this_platform=platform_desc($vars{'this_platform'});
 
-  my $b_path=button($vars{'option_path'});
-  my $b_doc=button($vars{'option_doc'});
-  my $b_src=button($vars{'option_src'});
-  my $b_fmt=button($vars{'option_fmt'});
-  my $b_letter=button($vars{'option_letter'});
-  my $b_deskint=button($vars{'option_desktop_integration'});
-  my $b_admin=button($vars{'option_w32_multi_user'});
+  my $b_path=button($vars{'instopt_adjustpath'});
+  my $b_doc=button($vars{'tlpdbopt_install_docfiles'});
+  my $b_src=button($vars{'tlpdbopt_install_srcfiles'});
+  my $b_fmt=button($vars{'tlpdbopt_create_formats'});
+  my $b_letter=button($vars{'instopt_letter'});
+  my $b_deskint=button($vars{'tlpdbopt_desktop_integration'}, \@::deskintdesc);
+  if (win32()) {
+    my $b_fileassoc=button($vars{'tlpdbopt_file_assocs'}, \@::fileassocdesc);
+  }
+  my $b_admin=button($vars{'tlpdbopt_w32_multi_user'});
   my $b_addoneditor=button($vars{'collection-texworks'});
-  my $b_restricted=button($vars{'option_write18_restricted'});
-  my $b_adjustrepo=button($vars{'option_adjustrepo'});
+  my $b_restricted=button($vars{'instopt_write18_restricted'});
+  my $b_adjustrepo=button($vars{'instopt_adjustrepo'});
 
   my $warn_nobin;
 
@@ -983,8 +1028,9 @@ sub main_menu {
     'O' => \&options_menu,
     'Q' => \&quit,
     'V' => \&toggle_portable,
+    'P' => \&callback_save_profile,
   );
-  if (!$vars{'in_place'}) {
+  if (!$opt_in_place) {
     $command{'B'} = \&binary_menu if unix();
     $command{'C'} = \&collection_menu;
     $command{'S'} = \&scheme_menu;
@@ -1001,7 +1047,7 @@ sub main_menu {
  $warn_nobin
 EOF
 
-  if (!$vars{'in_place'}) {
+  if (!$opt_in_place) {
     print <<"EOF";
  <B> set binary platforms: $vars{'n_systems_selected'} out of $vars{'n_systems_available'}
 
@@ -1051,7 +1097,7 @@ print <<"EOF";
    $b_fmt create all format files
 EOF
 
-  if (!$vars{'in_place'}) {
+  if (!$opt_in_place) {
     if ($vars{'doc_splitting_supported'}) {
       print "   $b_doc install macro/font doc tree\n";
     }
@@ -1060,10 +1106,10 @@ EOF
     }
   }
   if (win32()) {
-    if (!$vars{'portable'}) {
+    if (!$vars{'instopt_portable'}) {
       print "   $b_path adjust search path\n";
       print "   $b_deskint add menu items, shortcuts, etc.\n";
-      print "   [$vars{'option_file_assocs'}] update file associations\n";
+      print "   [$vars{'tlpdbopt_file_assocs'}] update file associations\n";
       if (admin()) {
         print "   $b_admin make installation available to all users\n";
       }
@@ -1072,19 +1118,19 @@ EOF
   }
 
   print "   $b_path create symlinks to standard directories\n"
-    unless ($vars{'portable'} || win32());
+    unless ($vars{'instopt_portable'} || win32());
   
   print "   $b_adjustrepo after install, use tlnet on CTAN "
           . "for package updates\n"
     unless ($media eq 'NET');
 
-  if ($vars{'portable'}) {
+  if ($vars{'instopt_portable'}) {
     print "\n <V> set up for regular installation to hard disk\n";
   } else {
     print "\n <V> set up for portable installation\n";
   }
 
-  other_options qw(I H Q);
+  other_options qw(I P H Q);
   my $answer = prompt 'Enter command';
 
   if (defined $command{"\u$answer"}) {
@@ -1094,7 +1140,6 @@ EOF
     return $command{'self'};
   }
 }
-
 # needs a terminal 1 for require to succeed!
 1;
 
